
 precision highp float;

 varying highp vec2 textureCoordinate;                  // vertex传来的纹理坐标
 uniform sampler2D vTexture;                   //sampler2D：纹理。vTexture ：application 规定并传来

 uniform highp float scale;                // 缩放系数，0无缩放，大于0则放大
 uniform highp float radius;                    // 缩放算法的作用域半径
 uniform highp vec2 leftEyeCenterPosition;      // 左眼控制点，越远变形越小
 uniform highp vec2 rightEyeCenterPosition;     // 右眼控制点
 uniform float aspectRatio;                     // 所处理图像的宽高比

 uniform mat4 matrix_Center;                        //纹理坐标点变化矩阵


 highp vec2 warpPositionToUse(vec2 pCenterPostion, vec2 currentPosition, float radius, float scale, float aspectRatio)
 {

     //坐标矩阵变换
     vec2 centerPostion =( matrix_Center * vec4( pCenterPostion , 0, 1) ).xy ;

     // 归一化坐标空间。需要考虑屏幕是一个单位宽和一个单位长。纹理坐标(0,1),是压缩长边，还原宽高比图像计算距离更准确
     vec2 currentPositionToUse = vec2(currentPosition.x, currentPosition.y * aspectRatio + 0.5 - 0.5 * aspectRatio); //归一化坐标空间需要考虑屏幕是一个单位宽和一个单位长。
     vec2 centerPostionToUse = vec2(centerPostion.x, centerPostion.y * aspectRatio + 0.5 - 0.5 * aspectRatio);

//.........输出黑点。调试用...................................
//         float dist = distance(currentPositionToUse, centerPostionToUse);
//         if (dist < 0.02f){
//             return vec2(99,99);
//         }
//............................................

     // 纹理坐标(0,1),是压缩长边，还原宽高比图像计算距离更准确
     float r = distance(currentPositionToUse, centerPostionToUse);

     vec2 positionToUse = currentPosition;

     if(r < radius)
     {
         float alpha = 1.0 - scale * pow(r / radius - 1.0, 2.0);
         positionToUse = centerPostion + alpha * (currentPosition - centerPostion);
     }

     return positionToUse;
 }

 void main()
 {
     vec2 positionToUse = warpPositionToUse(leftEyeCenterPosition, textureCoordinate, radius, scale, aspectRatio);
     positionToUse = warpPositionToUse(rightEyeCenterPosition, positionToUse, radius, scale, aspectRatio);

     gl_FragColor = texture2D(vTexture, positionToUse);
 }


