 precision highp float;

 varying highp vec2 textureCoordinate;              // vertex传来的纹理坐标
 uniform sampler2D vTexture;                        //sampler2D：纹理。vTexture ：application 规定并传来


 uniform highp float radius;                        //作用半径
 uniform highp float aspectRatio;                   //视图的高宽比

 uniform mat4 matrix_Center;                        //纹理坐标点变化矩阵


//最多支持单边 MAX_CONTOUR_POINT_COUNT个控制点：8

 uniform float leftContourPoints[16];               //左边控制点 MAX_CONTOUR_POINT_COUNT*2
 uniform float rightContourPoints[16];              //右边控制点 MAX_CONTOUR_POINT_COUNT*2
 uniform float deltaArray[8];                       // deltaArray？？？？       最多支持单边MAX_CONTOUR_POINT_COUNT个控制点
 uniform int arraySize;                             //实际单边控制点数。左右控制点总数：arraySize*2, 且arraySize 小于等于MAX_CONTOUR_POINT_COUNT




 //                                 当前点             对称的两个控制点（左右）              作用半径
 highp vec2 warpPositionToUse(vec2 currentPoint, vec2 pContourPointA,  vec2 pContourPointB, float radius, float delta, float aspectRatio)
 {
     //坐标矩阵变换
    vec2 contourPointA =( matrix_Center * vec4( pContourPointA , 0, 1) ).xy ;
    vec2 contourPointB =( matrix_Center * vec4( pContourPointB , 0, 1) ).xy ;


     vec2 positionToUse = currentPoint;

     vec2 currentPointToUse = vec2(currentPoint.x, currentPoint.y * aspectRatio + 0.5 - 0.5 * aspectRatio);
     vec2 contourPointAToUse = vec2(contourPointA.x, contourPointA.y * aspectRatio + 0.5 - 0.5 * aspectRatio);

     float r = distance(currentPointToUse, contourPointAToUse);
     if(r < radius)
     {
         vec2 dir = normalize(contourPointB - contourPointA);
         float dist = radius * radius - r * r;
         float alpha = dist / (dist + (r-delta) * (r-delta));
         alpha = alpha * alpha;

         positionToUse = positionToUse - alpha * delta * dir;
     }

     return positionToUse;
 }


 void main()
 {
     vec2 positionToUse = textureCoordinate;

     for(int i = 0; i < arraySize; i++)
     {
         positionToUse = warpPositionToUse(positionToUse, vec2(leftContourPoints[i * 2], leftContourPoints[i * 2 + 1]), vec2(rightContourPoints[i * 2], rightContourPoints[i * 2 + 1]), radius, deltaArray[i], aspectRatio);
         positionToUse = warpPositionToUse(positionToUse, vec2(rightContourPoints[i * 2], rightContourPoints[i * 2 + 1]), vec2(leftContourPoints[i * 2], leftContourPoints[i * 2 + 1]), radius, deltaArray[i], aspectRatio);
     }

     gl_FragColor = texture2D(vTexture, positionToUse);
 }





